
/*
 * Copyright (C) 2015-2018 Alibaba Group Holding Limited
 */

#include "dpm.h"
#include "ls_osa.h"

int dpm_client_sample(void)
{
    dpm_stat_t stat;
    uint32_t otp_len;
    uint32_t otp_step;
    uint32_t index;
    uint64_t time_ms;
    uint8_t otp_base[24] = {0};
    uint8_t otp_data[24] = {0};

    ls_osa_msleep(1);

    time_ms = ls_osa_get_time_ms();

    index = time_ms % 100;

    do {
        time_ms += ls_osa_get_time_ms();
        otp_len = time_ms % (DPM_OTP_DATA_MAX_LEN + 1);
    } while(otp_len < DPM_OTP_DATA_MIN_LEN);

    do {
        time_ms += ls_osa_get_time_ms();
        otp_step = time_ms % (DPM_OTP_STEP_MAX_LEN + 1);
    } while(otp_step < DPM_OTP_STEP_MIN_LEN);

    stat = dpm_get_totp(time_ms/1000, index, otp_step, otp_len, otp_base);
    if (stat != DPM_SUCCESS) {
        ls_osa_print("dpm generate totp fail, 0x%x\n", stat);
        return -1;
    }

    stat = dpm_get_index(otp_base, otp_len, &index);
    if (stat != DPM_SUCCESS) {
        ls_osa_print("dpm get index fail, 0x%x\n", stat);
        return -1;
    }

    stat = dpm_get_totp(time_ms/1000, index, otp_step, otp_len, otp_data);
    if (stat != DPM_SUCCESS) {
        ls_osa_print("dpm generate totp fail, 0x%x\n", stat);
        return -1;
    }

    if (memcmp(otp_base, otp_data, otp_len)) {
        ls_osa_print("generated totp not equal!\n");
        ls_osa_print("otp_base: %s\n", otp_base);
        ls_osa_print("otp_data: %s\n", otp_data);
        return -1;
    }

    return 0;
}

int main()
{
    int ret;

    ls_osa_print("===========> DPM Client Test start.\n");
    ret = dpm_client_sample();
    if (ret < 0) {
        ls_osa_print("DPM client Sample Test Failed!\n");
        return -1;
    }
    ls_osa_print("<=========== DPM Client Sample End.\n\n");

    return 0;
}

